/**
 * DevExtreme (ui/sortable.js)
 * Version: 19.2.1 (build 19259-0307)
 * Build date: Mon Sep 16 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _renderer = require("../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _component_registrator = require("../core/component_registrator");
var _component_registrator2 = _interopRequireDefault(_component_registrator);
var _extend = require("../core/utils/extend");
var _draggable = require("./draggable");
var _draggable2 = _interopRequireDefault(_draggable);
var _dom = require("../core/utils/dom");
var _translator = require("../animation/translator");
var _translator2 = _interopRequireDefault(_translator);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}
var SORTABLE = "dxSortable",
    SOURCE_CLASS = "source",
    PLACEHOLDER_CLASS = "placeholder";
var Sortable = _draggable2.default.inherit({
    dropItem: function() {
        var $sourceElement = this._getSourceElement(),
            sourceDraggable = this._getSourceDraggable(),
            hasPlaceholderTemplate = !!sourceDraggable.option("placeholderTemplate"),
            isNotSortable = sourceDraggable.NAME !== this.NAME;
        if (hasPlaceholderTemplate || isNotSortable) {
            var dragInfo = this._dragInfo,
                $targetItemElement = dragInfo && dragInfo.targetItemPoint && dragInfo.targetItemPoint.$item;
            if (isNotSortable) {
                _translator2.default.resetPosition($sourceElement)
            }
            this._moveItem($sourceElement, $targetItemElement, !$targetItemElement)
        }
    },
    clearDragInfo: function() {
        this._dragInfo = null
    },
    movePlaceholder: function(e) {
        var itemPoints = this._dragInfo && this._dragInfo.itemPoints;
        if (!itemPoints) {
            return
        }
        var isVertical = "vertical" === this.option("itemOrientation"),
            axisName = isVertical ? "top" : "left",
            cursorPosition = isVertical ? e.pageY : e.pageX;
        for (var i = 0; i < itemPoints.length; i++) {
            var itemPoint = itemPoints[i],
                centerPosition = itemPoints[i + 1] && (itemPoint[axisName] + itemPoints[i + 1][axisName]) / 2;
            if (centerPosition > cursorPosition) {
                this._updatePlaceholderPosition(e, itemPoint);
                break
            } else {
                if (void 0 === centerPosition) {
                    this._updatePlaceholderPosition(e, itemPoint, true);
                    break
                }
            }
        }
    },
    resetPlaceholder: function() {
        if (this._$placeholderElement) {
            var needToRemove = !this._$placeholderElement.hasClass(this._addWidgetPrefix(SOURCE_CLASS));
            if (needToRemove) {
                this._$placeholderElement.remove()
            } else {
                this._togglePlaceholderClass(false)
            }
        }
        this._$placeholderElement = null
    },
    setupDraggingInfo: function() {
        if (this._dragInfo) {
            return
        }
        var $sourceElement = this._getSourceElement();
        this._dragInfo = {
            fromIndex: $sourceElement.index(),
            itemPoints: this._getItemPoints()
        }
    },
    _getDefaultOptions: function() {
        return (0, _extend.extend)(this.callBase(), {
            clone: true,
            filter: "> *",
            itemOrientation: "vertical",
            placeholderTemplate: void 0,
            onDragChange: null
        })
    },
    _init: function() {
        this.callBase()
    },
    _getPlaceholderTemplate: function() {
        var sourceDraggable = this._getSourceDraggable();
        return sourceDraggable.option("placeholderTemplate")
    },
    _createPlaceholder: function() {
        var sourceDraggable = this._getSourceDraggable(),
            $placeholderContainer = this._getSourceElement(),
            placeholderTemplate = this._getPlaceholderTemplate();
        if (placeholderTemplate) {
            $placeholderContainer = (0, _renderer2.default)("<div>");
            placeholderTemplate = this._getTemplate(placeholderTemplate);
            placeholderTemplate.render({
                container: (0, _dom.getPublicElement)($placeholderContainer)
            })
        } else {
            if (sourceDraggable.NAME !== this.NAME) {
                $placeholderContainer = $placeholderContainer.clone();
                _translator2.default.resetPosition($placeholderContainer)
            }
        }
        this._$placeholderElement = $placeholderContainer;
        this._togglePlaceholderClass(true);
        return $placeholderContainer
    },
    _togglePlaceholderClass: function(value) {
        this._$placeholderElement && this._$placeholderElement.toggleClass(this._addWidgetPrefix(PLACEHOLDER_CLASS), value)
    },
    _getItems: function() {
        var itemsSelector = this._getItemsSelector();
        return this.$element().find(itemsSelector).not("." + this._addWidgetPrefix(PLACEHOLDER_CLASS)).toArray()
    },
    _isValidPoint: function($items, itemPointIndex) {
        var hasPlaceholderTemplate = !!this.option("placeholderTemplate");
        if (!hasPlaceholderTemplate) {
            return true
        }
        var $draggableItem = this._getDraggableElement(),
            draggableItemIndex = $items.indexOf($draggableItem.get(0));
        return draggableItemIndex === -1 || itemPointIndex !== draggableItemIndex && itemPointIndex !== draggableItemIndex + 1
    },
    _getItemPoints: function() {
        var _this = this;
        var result = void 0,
            isVertical = "vertical" === this.option("itemOrientation"),
            $items = this._getItems();
        result = $items.map(function(item, index) {
            var offset = (0, _renderer2.default)(item).offset();
            return {
                left: offset.left,
                top: offset.top,
                index: index,
                $item: (0, _renderer2.default)(item),
                width: (0, _renderer2.default)(item).outerWidth(),
                height: (0, _renderer2.default)(item).outerHeight(),
                isValid: _this._isValidPoint($items, index)
            }
        });
        if (result.length) {
            var lastItem = result[result.length - 1];
            result.push({
                index: result.length,
                top: isVertical ? lastItem.top + lastItem.height : lastItem.top,
                left: !isVertical ? lastItem.left + lastItem.width : lastItem.left,
                isValid: this._isValidPoint($items, result.length)
            })
        } else {
            result.push({
                index: 0,
                isValid: true
            })
        }
        return result
    },
    _dragStartHandler: function(e) {
        this.callBase.apply(this, arguments);
        if (true === e.cancel) {
            return
        }
        this.setupDraggingInfo()
    },
    _togglePlaceholder: function(value) {
        this._$placeholderElement && this._$placeholderElement.toggle(value)
    },
    _updatePlaceholderPosition: function(e, itemPoint, isLastPosition) {
        var eventArgs = void 0,
            fromIndex = this._dragInfo.fromIndex,
            sourceDraggable = this._getSourceDraggable();
        this._dragInfo.toIndex = Math.max(sourceDraggable !== this || fromIndex >= itemPoint.index ? itemPoint.index : itemPoint.index - 1, 0);
        this._dragInfo.targetItemPoint = itemPoint;
        eventArgs = (0, _extend.extend)(this._getEventArgs(), {
            event: e
        });
        this._getAction("onDragChange")(eventArgs);
        if (eventArgs.cancel || void 0 === eventArgs.cancel && !itemPoint.isValid) {
            this._togglePlaceholder(false);
            return
        }
        var $placeholderElement = this._$placeholderElement || this._createPlaceholder();
        this._togglePlaceholder(true);
        this._moveItem($placeholderElement, itemPoint.$item, isLastPosition);
        this._dragInfo.itemPoints = this._getItemPoints()
    },
    _moveItem: function($targetItem, $item, isLastPosition) {
        if (isLastPosition) {
            this.$element().append($targetItem)
        } else {
            $targetItem.insertBefore($item)
        }
    },
    _getEventArgs: function() {
        var sourceElement = (0, _dom.getPublicElement)(this._getSourceElement()),
            fromIndex = this._dragInfo && this._dragInfo.fromIndex,
            toIndex = this._dragInfo && this._dragInfo.toIndex;
        return {
            fromIndex: fromIndex,
            toIndex: toIndex,
            sourceElement: sourceElement
        }
    },
    _getDragEndArgs: function() {
        return (0, _extend.extend)(this.callBase.apply(this, arguments), this._getEventArgs())
    },
    _optionChanged: function(args) {
        var name = args.name;
        switch (name) {
            case "onDragChange":
                this["_" + name + "Action"] = this._createActionByOption(name);
                break;
            case "itemOrientation":
                break;
            default:
                this.callBase(args)
        }
    }
});
(0, _component_registrator2.default)(SORTABLE, Sortable);
module.exports = Sortable;
module.exports.default = module.exports;
